#!/usr/bin/php

<?php
//
//	名称
//		やすらかシステム（サーバ）
//	機能
//		やすらかシステム（ノード）からのデータを解析しデータベースへ追加する。
//	バージョン
//		0.1.1
//	使い方
//		UNIXシェルから ./yasuraka_server.php を入力
//	設定方法
//		設定項目部の定数を変更
//

// 設定項目
define( "DB_NAME", "yasuraka_db");			// データベース名
define( "DB_USER", "yasuraka");				// ユーザ名
define( "DB_PASS", "piyopiyo");				// パスワード

define( "IP_ADDRESS", "192.168.11.11");		// 待ち受けIPアドレス
define( "PORT", 10000);						// 待ち受けポート番号

// 実行時間制限を無効化
set_time_limit(0);

// 自動フラッシュを有効化
ob_implicit_flush();

// エラー発生時に例外を発生
error_reporting(E_ALL);
function exception_error_handler( $severity, $message, $file, $line)  {
	if( !(error_reporting() & $severity) )	{
		return;
	}
	echo "例外コード：" . $severity . "\n";
	throw new ErrorException($message, 0, $severity, $file, $line);
}
set_error_handler("exception_error_handler");

// シグナルハンドラを設定
declare(ticks=1);
function sig_handler($signo)  {
	global $sock;
	global $abort;
	switch( $signo )  {
		case SIGTERM:
			echo "SIGTERMを受け取りました\n";
			exit;
			break;
		case SIGHUP:
			echo "SIGHUPを受け取りました\n";
			break;
		case SIGINT:
			echo "SIGINTを受け取りました\n";
			break;
		default:
			print "シグナルを受け取りました\n";
	}
}
pcntl_signal( SIGTERM, "sig_handler");
pcntl_signal( SIGHUP,  "sig_handler");
pcntl_signal( SIGINT,  "sig_handler");

// データベースへ接続
try {
	$handle_db = new PDO('mysql:host=localhost;dbname='.DB_NAME, DB_USER, DB_PASS);
} catch (PDOException $e ) {
	$handle_db = null;
	print "エラー：" . $e->getMessage();
	return;
}

// ソケットを作成
$sock = socket_create( AF_INET, SOCK_STREAM, SOL_TCP);		// 作成
socket_set_option( $sock, SOL_SOCKET, SO_REUSEADDR, 1);		// オプションを設定
socket_bind( $sock, IP_ADDRESS, PORT);						// 関連付け
socket_listen( $sock);										// 接続待ち

// 接続中のソケットを配列で保持(接続待ち用ソケットも含む)
$clients = array($sock);

// 起動メッセージを表示
echo "やすらかシステム(サーバ) Version 0.1.1\n";
echo "  IPアドレス=".IP_ADDRESS." ポート=".PORT."\n";
echo "  Ctrl-Cで終了します。\n\n";

// メインループ
while(true)  {
	try  {
		// socket_selectで$clientsが破壊されないようコピーを作成
		$clients_work = $clients;
		
		// すべてのクライアントからの受信を待ち受け
		if( $res = socket_select( $clients_work, $write, $except, NULL) < 1 )  {
			continue;
		}
		
		// 接続要求の場合
		if( in_array($sock, $clients_work) )  {
			// acceptしソケットを$clientsに追加
			$clients[] = $newsock = socket_accept($sock);
			// 接続メッセージを表示
			socket_write( $newsock, "Welcome to MAMORUKUN.\n" .
			"There are " . ( count($clients) - 1 ) . " client(s) connected to the server\n");
			socket_getpeername($newsock, $ip, $port);
			echo "クライアントが接続されました。 IPアドレス={$ip} ポート番号={$port}\n";
			// $clients_workから$sockを削除(接続要求を受け付けたから)
			$key = array_search($sock, $clients_work);
			unset($clients_work[$key]);
		}
		
		// 受信のあったソケット全てについてループ
		foreach( $clients_work as $read_sock )	{
			// 1024バイト受信する
			// クライアントから切断されているとエラーが発生するがメッセージは表示しない
			$data = @socket_read( $read_sock, 1024, PHP_NORMAL_READ);

			// 切断された場合の処理
			if( $data === false )  {	// 受信データがfalseか？
				// $clientsから当該のソケットを削除
				$key = array_search( $read_sock, $clients);
				unset($clients[$key]);
				echo "クライアントが切断されました。\n";
				continue;
			}
			
			// 接続待ちソケットの場合は何もしない
			if( $read_sock == $sock )  {
				continue;
			}
			
			// 登録されている装置か確認
			socket_getpeername($read_sock, $ip, $port);
			$query_instrument = $handle_db->prepare('SELECT id,room,ip_address FROM instruments WHERE ip_address=:ip_address ');
			$query_instrument->bindParam(':ip_address',$ip, PDO::PARAM_STR);
			$query_instrument->execute();
			$result_instrument = $query_instrument->fetchAll();
			if( count($result_instrument) == 0 )  {
				echo "未登録の装置からイベントを受信しました。IPアドレス={$ip}\n";
				continue;
			}
			// 受信データを [キー名]=[値] の形に分解
			$data_trimed = trim( $data,"\r\n;");						// 前後の空白,改行,;を削除
			$data_exploded_semicolon = explode( ";", $data_trimed);		// ;で分割
			$event_received = array();
			foreach( $data_exploded_semicolon as $event_string)  {
				$data_exploded_equal = explode( "=", $event_string);
				if( count($data_exploded_equal) != 2 )	{
					echo "受信データが不正です。 IPアドレス={$ip}\n";
					continue 2;
				}
				$event_received[$data_exploded_equal[0]]= $data_exploded_equal[1];
			}
			// 受信データ毎の処理
			foreach( $event_received as $key_name => $value )  {
				// 登録されているイベントか確認
				$query_event = $handle_db->prepare('SELECT id,name FROM events WHERE name=:name ');
				$query_event->bindParam(':name',$key_name, PDO::PARAM_STR);
				$query_event->execute();
				$result_event = $query_event->fetchAll();
				if( count($result_event) == 0 )  {
					echo "未登録のイベントを受信しました。IPアドレス={$ip} イベント名={$key_name} 値={$value}\n";
					continue;
				}
				// 履歴を追加
				$result_add = $handle_db->prepare('INSERT INTO histories (instrument_id,date,event_id,value) VALUES (:instrument_id, cast(now() as datetime), :event_id, :value)');
				$result_add->bindParam( ':instrument_id',	$result_instrument[0]['id'],	PDO::PARAM_STR);
				$result_add->bindParam( ':event_id',		$result_event[0]['id'],			PDO::PARAM_STR);
				$result_add->bindParam( ':value',			$value,							PDO::PARAM_STR);
				$result_add->execute();
				echo "イベントを登録しました。 IPアドレス={$ip} イベント名={$key_name} 値={$value}\n";
			}
		}
	} catch( Exception $e ) {
		echo "例外: " . socket_strerror(socket_last_error()) ." at Line=" . $e->getLine() . "\n";
		socket_clear_error($sock);
		$handle_db = NULL;
		break;
	}
}

// データベースを切断
$handle_db = NULL;

// ソケットを閉じる
socket_close($sock);

?>
